/*
 * Copyright (C) 2025 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.aggregation.transformation;

import java.util.Optional;

import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.node.ObjectNode;
import com.schibsted.spt.data.jslt.Expression;
import com.schibsted.spt.data.jslt.impl.ExpressionNode;
import com.schibsted.spt.data.jslt.impl.Scope;

import de.ozgcloud.aggregation.warehouse.DocumentEntry;
import de.ozgcloud.apilib.vorgang.OzgCloudVorgang;
import lombok.Builder;

/**
 * Transformation based on {@link Expression}
 */
@Builder
public class JSLTransformation implements Transformation {

	private final Scope root = Scope.getRoot(0);
	private final ObjectMapper objectMapper;
	private final VorgangMapper vorgangMapper;
	@Builder.Default
	private final Optional<? extends ExpressionNode> identifier = Optional.empty();
	private final Expression script;

	@Override
	public DocumentEntry apply(OzgCloudVorgang vorgang) throws TransformationException {
		try {
			var builder = vorgangMapper.toDocumentEntryBuilder(vorgang, DocumentEntry.builder());
			var vorgangTree = objectMapper.valueToTree(vorgang);
			return overrideId(vorgangTree, builder)
					.payload((ObjectNode) script.apply(vorgangTree))
					.build();
		} catch (Exception e) {
			throw new TransformationException("unable to apply transformation", e);
		}
	}

	DocumentEntry.DocumentEntryBuilder overrideId(JsonNode tree, DocumentEntry.DocumentEntryBuilder documentBuilder) {
		identifier.ifPresent(expression -> documentBuilder.id(expression.apply(root, tree).asText()));
		return documentBuilder;

	}
}
