/*
 * Copyright (C) 2025 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.aggregation.transformation;

import de.ozgcloud.aggregation.warehouse.DocumentEntry;
import de.ozgcloud.apilib.vorgang.OzgCloudVorgang;

/**
 * A {@link Transformation} holds the data to transform {@link OzgCloudVorgang}
 * objects into {@link DocumentEntry}
 */
public interface Transformation {

	/**
	 * Apply this {@link Transformation} to a given {@link OzgCloudVorgang} to
	 * retrieve the related {@link DocumentEntry}
	 *
	 * @param vorgang {@link OzgCloudVorgang} to transform
	 * @return {@link DocumentEntry} to store
	 * @throws TransformationException when the {@link Transformation} could not be
	 *                                 applied to the given {@link OzgCloudVorgang}
	 */
	DocumentEntry apply(OzgCloudVorgang vorgang) throws TransformationException;

}
