/*
 * Copyright (C) 2025 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.aggregation.transformation;

import static org.assertj.core.api.Assertions.*;

import java.util.List;

import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Test;
import org.mapstruct.factory.Mappers;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.datatype.jsr310.JavaTimeModule;

import de.ozgcloud.aggregation.transformation.AggregationMapping.FieldMapping;
import de.ozgcloud.apilib.vorgang.OzgCloudVorgangTestFactory;

@DisplayName("JSLT transformations")
class JSLTServiceITCase {

	private static final String CREATED_AT_TARGET = "date";
	private static final String ID_TARGET = "internalId";
	private static final String FIELD_CREATED_AT = "header.createdAt";
	private static final String FIELD_ID = "id.value";

	@Test
	@DisplayName("Test simple transformation")
	void shouldApplySimpleTransformation() throws Exception {
		var objectMapper = configureMapper();
		var vorgangMapper = Mappers.getMapper(VorgangMapper.class);
		var fieldMappings = List.of(
				FieldMapping.builder()
						.sourcePath(FIELD_ID)
						.targetPath(ID_TARGET)
						.build(),
				FieldMapping.builder()
						.sourcePath(FIELD_CREATED_AT)
						.targetPath(CREATED_AT_TARGET)
						.build());
		var service = new JSLTransformationService(objectMapper, vorgangMapper);

		var transformation = service.load(null,
				AggregationMappingTestFactory.createBuilder().clearFieldMappings().fieldMappings(fieldMappings).build());

		var vorgang = OzgCloudVorgangTestFactory.create();

		var entry = transformation.apply(vorgang);

		assertThat(entry.getId()).isEqualTo(OzgCloudVorgangTestFactory.ID.toString());
		assertThat(entry.getPayload().get(ID_TARGET).asText()).isEqualTo(OzgCloudVorgangTestFactory.ID.toString());
		assertThat(entry.getPayload().get(CREATED_AT_TARGET).asLong()).isEqualTo(vorgang.getHeader().getCreatedAt().toEpochSecond());
	}

	private ObjectMapper configureMapper() {
		final ObjectMapper mapper = new ObjectMapper();
		mapper.registerModule(new JavaTimeModule());
		return mapper;
	}

}
