package de.ozgcloud.aggregation.warehouse;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.util.List;

import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;
import org.springframework.data.mongodb.core.MongoTemplate;

import com.thedeanda.lorem.LoremIpsum;

class CustomWarehouseRepositoryImplTest {

	@Spy
	@InjectMocks
	private CustomWarehouseRepositoryImpl repository;

	@Mock
	private MongoTemplate mongoTemplate;

	@Nested
	class TestSaveAllInCollection {

		private final DocumentEntry documentEntry = DocumentEntryTestFactory.create();
		private final List<DocumentEntry> documentEntries = List.of(documentEntry);
		private final String collectionName = LoremIpsum.getInstance().getWords(1);

		@Test
		void shouldCallSaveInCollection() {
			repository.saveAllInCollection(documentEntries, collectionName);

			verify(repository).saveInCollection(documentEntry, collectionName);
		}

		@Test
		void shouldReturnSavedDocumentEntry() {
			var savedDocumentEntry = DocumentEntryTestFactory.create();
			doReturn(savedDocumentEntry).when(repository).saveInCollection(any(), any());

			var returnedDocumentEntries = repository.saveAllInCollection(documentEntries, collectionName);

			assertThat(returnedDocumentEntries).containsExactly(savedDocumentEntry);
		}
	}

	@Nested
	class TestSaveInCollection {

		private final DocumentEntry documentEntry = DocumentEntryTestFactory.create();
		private final String collectionName = LoremIpsum.getInstance().getWords(1);

		@Test
		void testSaveInCollection() {
			repository.saveInCollection(documentEntry, collectionName);

			verify(mongoTemplate).save(documentEntry, collectionName);
		}

		@Test
		void shouldReturnSavedDocumentEntry() {
			var savedDocumentEntry = DocumentEntryTestFactory.create();
			when(mongoTemplate.save(documentEntry, collectionName)).thenReturn(savedDocumentEntry);

			var returnedDocumentEntry = repository.saveInCollection(documentEntry, collectionName);

			assertThat(returnedDocumentEntry).isEqualTo(savedDocumentEntry);
		}
	}

	@Nested
	class TestDeleteCollection {

		private final String collectionName = LoremIpsum.getInstance().getWords(1);

		@Test
		void shouldDropCollection() {
			repository.clearCollection(collectionName);

			verify(mongoTemplate).dropCollection(collectionName);
		}
	}
}
