package de.ozgcloud.aggregation.data;

import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Component;

import io.grpc.stub.StreamObserver;
import lombok.RequiredArgsConstructor;

@RequiredArgsConstructor
@Component
@Scope("prototype")
class GrpcAggregationDataRequestObserver implements StreamObserver<GrpcSendAggregationDataRequest> {

	private final AggregationDataMapper aggregationDataMapper;
	private final AggregationDataService aggregationDataService;

	private StreamObserver<GrpcSendAggregationDataResponse> responseObserver;

	public GrpcAggregationDataRequestObserver withResponseObserver(StreamObserver<GrpcSendAggregationDataResponse> responseObserver) {
		this.responseObserver = responseObserver;
		return this;
	}

	@Override
	public void onNext(GrpcSendAggregationDataRequest request) {
		request.getAggregationDataList().stream().map(aggregationDataMapper::fromGrpc)
				.forEach(data -> aggregationDataService.save(data.toBuilder().kommune(request.getMandant()).build(), request.getName()));
	}

	@Override
	public void onError(Throwable t) {
		responseObserver.onError(t);
	}

	@Override
	public void onCompleted() {
		responseObserver.onNext(GrpcSendAggregationDataResponse.getDefaultInstance());
		responseObserver.onCompleted();
	}

}
