package de.ozgcloud.aggregation.data;

import java.util.Map;
import java.util.Map.Entry;
import java.util.stream.Collectors;

import org.mapstruct.Mapper;

@Mapper
interface GrpcObjectMapper {

	default Map<String, Object> toMap(GrpcObject object) {
		return object.getPropertiesList().stream().map(this::mapProperty).collect(
				Collectors.toMap(Entry::getKey, Entry::getValue));

	}

	default Entry<String, Object> mapProperty(GrpcProperty property) {
		return Map.entry(property.getKey(), mapElement(property.getValue()));
	}

	default Object mapElement(GrpcElement element) {
		if (element.hasStringValue()) {
			return element.getStringValue();
		}
		if (element.hasLongValue()) {
			return element.getLongValue();
		}
		if (element.hasDoubleValue()) {
			return element.getDoubleValue();
		}
		if (element.hasBoolValue()) {
			return element.getBoolValue();
		}
		if (element.hasObjectValue()) {
			return toMap(element.getObjectValue());
		}
		if (element.hasListValue()) {
			return element.getListValue().getElementsList().stream()
					.map(this::mapElement).toList();
		}
		return null;
	}
}
