package de.ozgcloud.aggregation.data;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

import io.grpc.stub.StreamObserver;

class AggregationDataGrpcServiceTest {

	@InjectMocks
	@Spy
	private AggregationDataGrpcServiceImpl service;
	@Mock
	private static GrpcAggregationDataRequestObserver grpcAggregationDataRequestObserver;

	@Nested
	class TestSendAggregationData {

		@Mock
		private StreamObserver<GrpcSendAggregationDataResponse> responseObserver;
		@Mock
		private GrpcAggregationDataRequestObserver grpcAggregationDataRequestObserverWithRepsonseObserver;

		@BeforeEach
		void mock() {
			when(grpcAggregationDataRequestObserver.withResponseObserver(any())).thenReturn(grpcAggregationDataRequestObserverWithRepsonseObserver);
		}

		@Test
		void shouldGetGrpcAggregationDataRequestObserver() {
			service.sendAggregationData(responseObserver);

			verify(service).getGrpcAggregationDataRequestObserver();
		}

		@Test
		void shouldSetResponseObserver() {
			service.sendAggregationData(responseObserver);

			verify(grpcAggregationDataRequestObserver).withResponseObserver(responseObserver);
		}

		@Test
		void shouldReturnRequestObserverWithResponseObserver() {
			var requestObserver = service.sendAggregationData(responseObserver);

			assertThat(requestObserver).isSameAs(grpcAggregationDataRequestObserverWithRepsonseObserver);
		}
	}

	static class AggregationDataGrpcServiceImpl extends AggregationDataGrpcService {

		@Override
		GrpcAggregationDataRequestObserver getGrpcAggregationDataRequestObserver() {
			return grpcAggregationDataRequestObserver;
		}
	}
}
