package de.ozgcloud.aggregation.data;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mapstruct.factory.Mappers;
import org.mockito.InjectMocks;
import org.mockito.Mock;

class AggregationDataMapperTest {

	@InjectMocks
	private final AggregationDataMapper aggregationDataMapper = Mappers.getMapper(AggregationDataMapper.class);

	@Mock
	private GrpcObjectMapper grpcObjectMapper;

	@Nested
	class TestFromGrpc {

		@Test
		void shouldMapHeaderData() {
			var aggregationData = aggregationDataMapper.fromGrpc(GrpcAggregationDataTestFactory.create());

			assertThat(aggregationData).usingRecursiveComparison()
					.ignoringFields("payload", "kommune")
					.isEqualTo(AggregationDataTestFactory.create());
		}

		@Test
		void shouldCallGrpcObjectMapper() {
			aggregationDataMapper.fromGrpc(GrpcAggregationDataTestFactory.create());

			verify(grpcObjectMapper).toMap(GrpcAggregationDataTestFactory.PAYLOAD);
		}

		@Test
		void shouldSetPayload() {
			when(grpcObjectMapper.toMap(any())).thenReturn(AggregationDataTestFactory.PAYLOAD);

			var aggregationData = aggregationDataMapper.fromGrpc(GrpcAggregationDataTestFactory.create());

			assertThat(aggregationData.getPayload()).isEqualTo(AggregationDataTestFactory.PAYLOAD);
		}
	}
}
