package de.ozgcloud.aggregation.data;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.springframework.test.util.ReflectionTestUtils;

import io.grpc.stub.StreamObserver;

class GrpcAggregationDataRequestObserverTest {

	@InjectMocks
	private GrpcAggregationDataRequestObserver grpcAggregationDataRequestObserver;

	@Mock
	private AggregationDataMapper aggregationDataMapper;
	@Mock
	private AggregationDataService aggregationDataService;
	@Mock
	private StreamObserver<GrpcSendAggregationDataResponse> responseObserver;

	@Nested
	class TestWithResponseObserver {

		@Test
		void shouldSetResponseObserver() {
			var requestObserver = grpcAggregationDataRequestObserver.withResponseObserver(responseObserver);

			assertThat(ReflectionTestUtils.getField(requestObserver, GrpcAggregationDataRequestObserver.class, "responseObserver"))
					.isEqualTo(responseObserver);
		}
	}

	@Nested
	class TestOnNext {

		@BeforeEach
		void mock() {
			when(aggregationDataMapper.fromGrpc(any())).thenReturn(AggregationDataTestFactory.createBuilder().kommune(null).build());
		}

		@Test
		void shouldCallAggregationDataMapper() {
			grpcAggregationDataRequestObserver.onNext(GrpcSendAggregationDataRequestTestFactory.create());

			verify(aggregationDataMapper).fromGrpc(GrpcSendAggregationDataRequestTestFactory.AGGREGATION_DATA);
		}

		@Test
		void shouldCallAggregationDataService() {
			grpcAggregationDataRequestObserver.onNext(GrpcSendAggregationDataRequestTestFactory.create());

			verify(aggregationDataService).save(AggregationDataTestFactory.create(), GrpcSendAggregationDataRequestTestFactory.MAPPING_NAME);
		}
	}

	@Nested
	class TestOnError {

		@BeforeEach
		void init() {
			grpcAggregationDataRequestObserver.withResponseObserver(responseObserver);
		}

		@Test
		void shouldCallResponseObserverOnError() {
			var error = new RuntimeException("Test error");

			grpcAggregationDataRequestObserver.onError(error);

			verify(responseObserver).onError(error);
		}
	}

	@Nested
	class TestOnCompleted {

		@BeforeEach
		void init() {
			grpcAggregationDataRequestObserver.withResponseObserver(responseObserver);
		}

		@Test
		void shouldCallOnNext() {
			grpcAggregationDataRequestObserver.onCompleted();

			verify(responseObserver).onNext(GrpcSendAggregationDataResponse.getDefaultInstance());
		}

		@Test
		void shouldCallResponseObserverOnCompleted() {
			grpcAggregationDataRequestObserver.onCompleted();

			verify(responseObserver).onCompleted();
		}
	}

}
